<?php

if ( ! defined( 'ABSPATH' ) ) {
    die( 'No direct access.' );
}

/**
 * Local Video Slide
 */
class MetaLocalVideoSlide extends MetaSlide
{

    public $identifier = 'local_video'; // should be lowercase, one word (use underscores if needed)
    public $name;


    /**
     * Register slide type
     */
    public function __construct()
    {
        $this->name = __( 'Local Video', 'ml-slider-pro' );

        if ( is_admin() ) {
            add_action( "wp_ajax_create_{$this->identifier}_slide", array( $this, 'ajax_create_slide' ) );
            add_filter( 'media_view_strings', array( $this, 'custom_media_uploader_tabs' ), 10, 1 );
            add_action( 'metaslider_register_admin_scripts', array( $this, 'register_admin_scripts' ), 10, 1 );
            add_action( 'metaslider_register_admin_styles', array( $this, 'register_admin_styles' ), 10, 1 );
        }

        add_action( 'wp_ajax_update_slide_video', array( $this, 'ajax_update_slide_video' ) );
        add_action( "metaslider_save_{$this->identifier}_slide", array( $this, 'save_slide' ), 5, 3 );
        add_filter( "metaslider_get_{$this->identifier}_slide", array( $this, 'get_slide' ), 10, 2 );
    }

    /**
     * Creates a new media manager tab
     *
     * @param array $strings registered media manager tabs
     *
     * @return array
     */
    public function custom_media_uploader_tabs( $strings )
    {
        $strings['insertLocalVideo'] = __( 'Local Video', 'ml-slider-pro' );
        return $strings;
    }

    /**
     * Registers and enqueues admin JavaScript
     */
    public function register_admin_scripts()
    {
        wp_enqueue_script(
            'metasliderpro-local-video-script',
            plugins_url( 'assets/local_video.js', __FILE__ ),
            array( 'metaslider-admin-script' ),
            METASLIDERPRO_VERSION,
            true
        );

        // Nonce loaded through metasliderpro-local-video-script localized script
        wp_localize_script( 
            'metasliderpro-local-video-script', 
            'metaslider_local_video', 
            array( 
                'nonce' => wp_create_nonce( 'metaslider_create_local_video_nonce' ),
                'update_slide_nonce' => wp_create_nonce( 'metaslider_update_local_video_nonce' ),
                'update_video_text' => esc_html__( 'Select replacement video', 'ml-slider' ),
                'update_cover_text' => esc_html__( 'Select replacement cover', 'ml-slider' )
            )
        );
    }

    /**
     * Registers and enqueues admin Styles
     */
    public function register_admin_styles()
    {
        $css = '.update-cover-image {
            width: 150px;
            height: 117px;
            background-color: #f0f0f0;
            color: #1E1E1E;
            background-size: cover;
            background-position: center;
        }
        .local_video_slide video {
            object-fit: cover;
            height: 100%;
            width: 100%;
        }';
        wp_add_inline_style( 'metaslider-admin-styles', $css );
    }

    /**
     * Extract the slide setings
     *
     * @param integer $id Slide ID
     */
    public function set_slide( $id )
    {
        parent::set_slide( $id );
        $this->slide_settings = get_post_meta( $id, 'ml-slider_settings', true );
    }

    /**
     * Create a new local video slide.
     */
    public function ajax_create_slide()
    {
        if ( ! isset($_POST['nonce'] ) || ! wp_verify_nonce( sanitize_key( $_POST['nonce'] ), 'metaslider_create_local_video_nonce' ) ) {
            wp_send_json_error( esc_html__( 'Invalid nonce', 'ml-slider-pro' ), 403 );
        }

        if ( ! isset( $_POST['slider_id'] ) || ! isset( $_POST['video_id'] ) ) {
            wp_send_json_error( esc_html__( 'Bad request', 'ml-slider-pro' ), 400 );
        }

        $video_id   = intval( $_POST['video_id'] );
        $slider_id  = intval( $_POST['slider_id'] );

        // Only allow mp4, webm and mov videos
        if( ! $this->video_type_is_allowed( $video_id ) ) {
            wp_send_json_error( 
                esc_html__( 'Not supported video format', 'ml-slider-pro' ),
                422
            );
        }

        // Set the slideshow (that this slide belongs to)
        $this->set_slider( $slider_id );

        // Create a new post for a slide
        $new_slide_id = $this->insert_slide( $video_id, 'local_video', $slider_id );

        // Save video id postmeta
        if( wp_attachment_is( 'video', $video_id ) ) {
            update_post_meta( $new_slide_id, 'ml-slider_video_id', $video_id );
        }

        // Set the slide
        $this->set_slide( $new_slide_id );

        // Tag the slide attachment to the slider tax category
        $this->tag_slide_to_slider();

        // finally, return the admin table row HTML
        // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
        echo $this->get_admin_slide();

        wp_die();
    }

    /**
     * Check if video type is allowed (mp4, webm and mov)
     * 
     * @param integer $video_id Video ID from Media library
     * 
     * @return boolean
     */
    protected function video_type_is_allowed( $video_id )
    {
        $allowed = array(
            'video/quicktime',
            'video/mp4',
            'video/webm'
        );

        if( ! in_array( get_post_mime_type( $video_id ), $allowed ) ) {
            return false;
        }

        return true;
    }

    /**
     * Return the admin slide HTML
     *
     * @return string
     */
    protected function get_admin_slide()
    {
        $video_url      = $this->get_video();
        $video_id       = $this->get_video_id();
        $video_type     = get_post_mime_type( $video_id );

        if( isset( $video_url ) && ! empty( $video_url ) ) {
            $video_thumb  = '<video loop muted';
            $video_thumb .= '   onmouseover="this.play()" onmouseout="this.pause()">';
            $video_thumb .= '    <source src="' . esc_url( $video_url ) . '"';
            $video_thumb .= '       type="' . esc_attr( $video_type ) . '"></source>';
            $video_thumb .= '</video>';
        } else {
            $video_thumb = '';
        }

        ob_start();
        // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
        echo $this->get_delete_button_html();
        // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
        echo "<button class='toolbar-button update-video alignright tipsy-tooltip-top' data-slide-type='local_video' data-button-text='" . esc_attr__( 'Update slide video', 'ml-slider' ) . "' title='" . esc_attr__( 'Update slide video', 'ml-slider' ) . "' data-slide-id='" . esc_attr( $this->slide->ID ) . "' data-attachment-id='" . esc_attr( $video_id ) . "'><i><svg xmlns='http://www.w3.org/2000/svg' width='20' height='20' viewBox='0 0 24 24' fill='none' stroke='currentColor' stroke-width='2' stroke-linecap='round' stroke-linejoin='round' class='feather feather-edit-2'><polygon points='16 3 21 8 8 21 3 21 3 16 16 3'/></svg></i></button>";
        do_action('metaslider-slide-edit-buttons', $this->identifier, $this->slide->ID);
        $edit_buttons = ob_get_clean();

        

        $row = "<tr id='slide-" . esc_attr( $this->slide->ID ) . "' class='slide local_video_slide flex'>";
        $row .= "    <td class='col-1'>";
        $row .= "       <div class='metaslider-ui-controls ui-sortable-handle'>";
        $row .= "           <h4 class='slide-details'>";
        $row .= esc_html__( 'Local Video Slide', 'ml-slider-pro' ) . " | ID: ". esc_html( $this->slide->ID );
        $row .= "           </h4>";
        if ( metaslider_this_is_trash( $this->slide ) ) {
            $row .= '<div class="row-actions trash-btns">';
            $row .= "<span class='untrash'>{$this->get_undelete_button_html()}</span>";
            if( method_exists( $this, 'get_permanent_delete_button_html' ) ) {
                $row .= ' | ';
                $row .= "<span class='delete'>{$this->get_permanent_delete_button_html()}</span>";
            }
            $row .= '</div>';
        } else {
            $row .= $edit_buttons;
        }
        $row .= "       </div>";
        $row .= "       <div class='metaslider-ui-inner'>";
        $row .= "           <button class='update-video image-button' data-button-text='" . esc_attr__(
                "Update slide video",
                "ml-slider"
            ) . "' title='" . esc_attr__(
                "Update slide video",
                "ml-slider"
            ) . "' data-slide-id='" . esc_attr( $this->slide->ID ) . "'";
        $row .= " data-attachment-id='" . esc_attr( $video_id ) . "'";
        $row .= " data-slide-type='local_video'>";
        $row .= "               <div class='thumb'>";
        $row .=                     $video_thumb;
        $row .= "               </div>";
        $row .= "           </button>";
        $row .= "       </div>";
        $row .= "    </td>";
        $row .= "    <td class='col-2'>";
        $row .= "       <div class='metaslider-ui-inner flex flex-col h-full'>";

        if ( method_exists( $this, 'get_admin_slide_tabs_html' ) ) {
            $row .= $this->get_admin_slide_tabs_html();
        } else {
            $row .= "<p>" . esc_html__( 'Please update to MetaSlider to version 3.2 or above.', 'ml-slider-pro' ) . "</p>";
        }

        $row .= "        <input type='hidden' name='attachment[" . esc_attr( $this->slide->ID ) . "][type]' value='local_video' />";
        $row .= "        <input type='hidden' name='attachment[" . esc_attr( $this->slide->ID ) . "][menu_order]' class='menu_order' value='" . esc_attr( $this->slide->menu_order ) . "' />";
        $row .= "        <input type='hidden' name='resize_slide_id' data-slide_id='" . esc_attr( $this->slide->ID ) . "' data-width='" . esc_attr( $this->settings['width'] ) . "' data-height='" . esc_attr( $this->settings['height'] ) . "' />";
        $row .= "       </div>";
        $row .= "    </td>";
        $row .= "</tr>";

        return $row;
    }

    /**
     * Build an array of tabs and their titles to use for the admin slide.
     */
    public function get_admin_tabs()
    {
        $slide_id   = absint( $this->slide->ID );
        $video_url  = $this->get_video();
        $video_id   = $this->get_video_id();
        $cover_id   = $this->get_attachment_id();

        $mute_checked = isset( $this->slide_settings['mute'] ) && filter_var(
            $this->slide_settings['mute'],
            FILTER_VALIDATE_BOOLEAN
        ) ? 'checked=checked' : '';
        $controls_checked = ! isset( $this->slide_settings['controls'] ) 
            || $this->slide_settings['controls'] == 'on' 
            ? 'checked=checked' : '';
        $autoPlay_checked = isset( $this->slide_settings['autoPlay'] ) && filter_var(
            $this->slide_settings['autoPlay'],
            FILTER_VALIDATE_BOOLEAN
        ) ? 'checked=checked' : '';
        $lazyLoad_checked = isset( $this->slide_settings['lazyLoad'] ) && filter_var(
            $this->slide_settings['lazyLoad'],
            FILTER_VALIDATE_BOOLEAN
        ) ? 'checked=checked' : '';
        $loop_checked = isset($this->slide_settings['loop']) && filter_var(
            $this->slide_settings['loop'],
            FILTER_VALIDATE_BOOLEAN
        ) ? 'checked=checked' : '';
        
        // General tab
        $general_tab  = '<input class="default url-readonly" type="text"';
        $general_tab .= ' data-attachment-id="' . esc_attr( $video_id ) . '"';
        $general_tab .= ' placeholder="' . esc_attr( 'Video File', 'ml-slider-pro' ) . '"';
        $general_tab .= ' value="' . esc_attr( $video_url ) . '"';
        $general_tab .= ' readonly />';

        // Hidden field with the post id from video
        $general_tab .= '<input type="hidden" name="attachment[' . esc_attr( $slide_id ) . '][video_id]"';
        $general_tab .= ' value="' . $video_id . '" />';

        $general_tab .= "<ul class='ms-split-li'>
                            <li><label><input type='checkbox' name='attachment[{$slide_id}][settings][mute]' {$mute_checked}/><span>" . __(
                'Mute video',
                'ml-slider-pro'
            ) . "</span></label></li>
                            <li><label><input type='checkbox' name='attachment[{$slide_id}][settings][controls]' {$controls_checked}/><span>" . __(
                'Show controls',
                'ml-slider-pro'
            ) . "</span></label></li>
                            <li><label><input type='checkbox' name='attachment[{$slide_id}][settings][autoPlay]' {$autoPlay_checked}/><span>" . __(
                'Auto play (may require video to be muted)&lrm;',
                'ml-slider-pro'
            ) . "</span></label></li>
                            <li><label><input type='checkbox' name='attachment[{$slide_id}][settings][lazyLoad]' {$lazyLoad_checked}/><span>" . __(
                'Lazy load video',
                'ml-slider-pro'
            ) . "</span></label></li>
                            <li><label><input type='checkbox' name='attachment[{$slide_id}][settings][loop]' {$loop_checked}/><span>" . __(
                'Loop video',
                'ml-slider-pro'
            ) . "</span></label></li>
                        </ul>";

        // Cover
        $cover_tab    = '<button class="update-cover-image"';
        $cover_tab   .= ' data-button-text="' . esc_attr__( 'Update cover image', 'ml-slider' ) . '"';
        $cover_tab   .= ' data-slide-id="' . esc_attr( $this->slide->ID ) . '"';
        $cover_tab   .= ' data-slide-type="local_video"';
        $cover_tab   .= ' data-attachment-id="' . esc_attr( $cover_id ) . '"';
        
        // Cover image if exists
        $thumb = $this->get_thumb();
        if( ! empty( $thumb ) ) {
            $cover_tab   .= ' title="' . esc_attr__( 'Update cover image', 'ml-slider' ) . '"';
            $cover_tab   .= ' style="background-image: url(' . esc_url( $thumb ) . ')"';
        }
        
        $cover_tab   .= '>';

        // Hide text if cover image is set
        if( empty( $thumb ) ) {
            $cover_tab   .= esc_html__( 'Set cover image', 'ml-slider-pro' );
        }

        $cover_tab   .= '</button>';

        $tabs = array(
            'general' => array(
                'title' => esc_html__( 'General', 'ml-slider-pro' ),
                'content' => $general_tab
            ),
            'cover' => array(
                'title' => esc_html__( 'Cover', 'ml-slider-pro' ),
                'content' => $cover_tab
            )
        );

        return apply_filters( 
            'metaslider_local_video_slide_tabs', 
            $tabs, 
            $this->slide, 
            $this->slider, 
            $this->settings 
        );
    }

    /**
     * Save
     *
     * @param array $fields Slide fields
     */
    protected function save( $fields )
    {
        // Update the order
        wp_update_post(array(
            'ID' => $this->slide->ID,
            'menu_order' => $fields['menu_order']
        ));
        
        /* We can add more fields to the array 
         * e.g. 'caption' which in db option would be saved as 'ml-slider_caption'
         */
        foreach( array( 'video_id' ) as $item ) {
            $this->add_or_update_or_delete_meta( $this->slide->ID, $item, $fields[$item] );
        }

        // Set defaults for non existing fields
        foreach ( array( 'mute', 'controls', 'autoPlay', 'lazyLoad', 'loop' ) as $setting ) {
            if ( ! isset($fields['settings'][$setting] ) ) {
                $fields['settings'][$setting] = 'off';
            }
        }

        // Save all the settings fields serialized
        if ( isset( $fields['settings'] ) ) {
            $this->add_or_update_or_delete_meta( $this->slide->ID, 'settings', $fields['settings'] );
        }
    }

    /**
     * Get the video for the slide
     */
    protected function get_video( $id = false )
    {
        $video_id = $id ? $id : $this->get_video_id();

        if( $video_id && wp_attachment_is( 'video', $video_id ) ) {
            $video_url = wp_get_attachment_url( $video_id );
        }

        if ( isset( $video_url ) ) {
            /*echo '----' . $video_id . '<br>';
            echo $video . '<br>';
            echo wp_attachment_is( 'video', $video_id ) . '<br>';*/
            return $video_url;
        }

        return '';
    }

    /**
     * Get the post id from a video
     */
    protected function get_video_id()
    {
        $video_id = get_post_meta( $this->slide->ID, 'ml-slider_video_id', true );

        if ( isset( $video_id ) ) {
            return absint( $video_id );
        }

        return false;
    }

    /**
     * Ajax wrapper to update the slide video.
     *
     * @return String The status message and if success, the thumbnail link (JSON)
     */
    public function ajax_update_slide_video()
    {
        if ( ! isset( $_REQUEST['_wpnonce'] ) 
            || ! wp_verify_nonce( sanitize_key( $_REQUEST['_wpnonce'] ), 'metaslider_update_local_video_nonce') ) {
            wp_send_json_error( array(
                'message' => __( 'The security check failed. Please refresh the page and try again.', 'ml-slider' )
            ), 401 );
        }

        $capability = apply_filters( 'metaslider_capability', MetaSliderPlugin::DEFAULT_CAPABILITY_EDIT_SLIDES );
        if ( ! current_user_can( $capability ) ) {
            wp_send_json_error(
                [
                    'message' => __( 'Access denied', 'ml-slider' )
                ],
                403
            );
        }

        if ( ! isset( $_POST['slide_id'] ) || ! isset($_POST['video_id']) || ! isset( $_POST['slider_id'] ) ) {
            wp_send_json_error(
                [
                    'message' => __( 'Bad request', 'ml-slider' ),
                ],
                400
            );
        }

        // Only allow mp4, webm and mov videos
        if( ! $this->video_type_is_allowed( absint( $_POST['video_id'] ) ) ) {
            wp_send_json_error( 
                [
                    'message' => esc_html__( 'Not supported video format', 'ml-slider-pro' )
                ],
                422
            );
        }

        $result = $this->update_slide_video(
            absint( $_POST['slide_id'] ),
            absint( $_POST['video_id'] ),
            absint( $_POST['slider_id'] )
        );

        if ( is_wp_error( $result ) ) {
            wp_send_json_error( array(
                'message' => $result->get_error_message()
            ), 409 );
        }
        wp_send_json_success ( $result, 200 );
    }

    /**
     * Updates the slide meta value to a new video.
     *
     * @param int $slide_id     The id of the slide being updated
     * @param int $video_id     The id of the new video to use
     * @param int $slideshow_id The id of the slideshow
     *
     * @return array|WP_error The status message and if success, the thumbnail link
     */
    protected function update_slide_video( $slide_id, $video_id, $slideshow_id = null )
    {   
        /*
         * Verifies that the $video_id is an actual video
         */
        if ( ! wp_attachment_is( 'video', $video_id ) ) {
            return new WP_Error( 
                'update_failed', 
                __( 'The requested video does not exist. Please try again.', 'ml-slider' ), 
                array( 'status' => 409 )
            );
        }

        /*
        * Updates database record and thumbnail if selection changed, assigns it to the slideshow, crops the image
        */
        update_post_meta( $slide_id, 'ml-slider_video_id', $video_id );
        if ( $slideshow_id ) {
            $this->set_slider( $slideshow_id );

            return array(
                'message' => __( 'The video was successfully updated.', 'ml-slider' ),
                'video_url' => $this->get_video( $video_id ),
                'video_type' => get_post_mime_type( $video_id )
            );
        }

        return new WP_Error( 
            'update_failed', 
            __( 'There was an error updating the video. Please try again', 'ml-slider' ), 
            array( 'status' => 409 ) 
        );
    }
    
    /**
     * Public slide html
     *
     * @return string
     */
    protected function get_public_slide()
    {
        wp_enqueue_script(
            'metasliderpro-videojs-script',
            METASLIDERPRO_BASE_URL . 'node_modules/video.js/dist/video.min.js',
            array(),
            METASLIDERPRO_VERSION,
            true
        );

        wp_enqueue_style(
            'metasliderpro-videojs-default-style',
            METASLIDERPRO_BASE_URL . 'node_modules/video.js/dist/video-js.min.css',
            array(),
            METASLIDERPRO_VERSION
        );

        add_action( 'metaslider_register_public_styles', array( $this, 'add_extra_styles' ), 10, 2 );

        $settings   = get_post_meta( $this->slider->ID, 'ml-slider_settings', true );
        $video_id   = get_post_meta( $this->slide->ID, 'ml-slider_video_id', true );
        $video_url  = $this->get_video( $video_id );

        if (! (int)$this->settings['height'] || ! (int)$this->settings['width']) {
            $ratio = 9 / 16 * 100;
        } else {
            $ratio = $this->settings['height'] / $this->settings['width'] * 100;
        }

        // Flexslider
        if ( $this->settings['type'] == 'flex' ) {
            add_filter( 'metaslider_flex_slider_parameters', array( $this, 'get_flex_slider_parameters' ), 10, 2 );

            return $this->get_flex_slider_markup( $video_id, $settings, $ratio );
        }
    }

    /**
     * Add inline styles used by videos
     */
    public function add_extra_styles()
    {
        $css  = '.metaslider .ms-local-video .play_button{position:absolute;top:0;left:0;width:100%;height:100%;display:flex;align-items:center;justify-content:center}';
        $css .= '.metaslider .ms-local-video .play_button img{width:75px;cursor:pointer;opacity:0.8}';
        $css .= '.metaslider .ms-local-video .play_button img:hover{opacity:1}';
        $css .= '.metaslider .ms-local-video div.video-js{position:absolute;top:0;left:0}';

        $css = apply_filters( 'metaslider_local_video_inline_css', $css, $this->slide, $this->slider->ID );
        wp_add_inline_style( 'metaslider-public', $css );
    }

    /**
     * Return the slide HTML for flex slider
     *
     * @return string
     */
    private function get_flex_slider_markup()
    {
        $html  = $this->video_wrapper();

        $attributes = array(
            'class' => "slide-{$this->slide->ID} ms-local-video",
            'style' => "display: none; width: 100%;"
        );

        $attributes = apply_filters(
            'metaslider_flex_slider_li_attributes',
            $attributes,
            $this->slide->ID,
            $this->slider->ID,
            $this->settings
        );

        $li = "<li";

        foreach ($attributes as $att => $val) {
            $li .= " " . $att . '="' . esc_attr($val) . '"';
        }

        $li .= ">" . $html . "</li>";

        $html = $li;

        return $html;
    }

    /**
     * Return the video wrapper with custom data attributes
     *
     * @return string
     */
    private function video_wrapper()
    {
        if ( ! (int) $this->settings['height'] || ! (int) $this->settings['width'] ) {
            $ratio = 9 / 16 * 100;
        } else {
            $ratio = $this->settings['height'] / $this->settings['width'] * 100;
        }

        $imageHelper = new MetaSliderImageHelper(
            $this->slide->ID,
            $this->settings['width'],
            $this->settings['height'],
            isset( $this->settings['smartCrop'] ) ? $this->settings['smartCrop'] : 'false'
        );
        
        $video_id       = $this->get_video_id();
        $poster         = $imageHelper->get_image_url();
        $controls       = ! isset( $this->slide_settings['controls'] ) || filter_var(
            $this->slide_settings['controls'],
            FILTER_VALIDATE_BOOLEAN
        ) ? 'true' : 'false';

        $attributes = array(
            'class'             => 'local-video video-js',
            'data-id'           => $this->slide->ID, // Slide ID, not video ID!
            'data-lazy-load'    => $this->get_setting_state( 'lazyLoad' ) ? 'true' : 'false',
            'data-url'          => $this->get_video(),
            'data-type'         => get_post_mime_type( $video_id ),
            'data-controls'     => $controls,
            'data-loop'         => $this->get_setting_state( 'loop' ) ? 'true' : 'false',
            'data-poster'       => $poster,
            'data-autoplay'     => $this->get_setting_state( 'autoPlay' ) ? 'true' : 'false',
            'data-mute'         => $this->get_setting_state( 'mute' ) ? 'true' : 'false',
            'style'             => 'position:relative;height:0;' . sprintf( 'padding-bottom:%s%%', $ratio )
        );

        $html = $attrs = "";
        foreach ( $attributes as $att => $val ) {
            $attrs .= " " . $att . '="' . esc_attr( $val ) . '"';
        }

        $html .= "<div{$attrs}>";
        
        if( $this->get_setting_state( 'lazyLoad' ) ) {
            $attachment_id  = $this->get_attachment_id();
            $alt            = esc_attr( get_post_meta( $attachment_id, '_wp_attachment_image_alt', true ) );
            
            $html .= $this->temporary_video_image( $poster, $alt );
        }

        $html .= "</div>";

        return $html;
    }

    /**
     * Get the title of the video
     *
     * @param int|string $url - Image URL
     * @param int|string $alt - Image alt
     * @return string
     */
    public function temporary_video_image( $url, $alt )
    {
        $html = "";

        if( ! empty( $url ) ) {
            $attributes = array(
                'src' => $url,
                'alt' => $alt,
                'title' => '',
                'class' => 'msDefaultImage',
                'height' => $this->settings['height'],
                'width' => $this->settings['width']
            );

            $html .= $this->build_image_tag( $attributes );
        }

        // Display button when controls are enabled
        if( ! isset( $this->slide_settings['controls'] ) || filter_var(
            $this->slide_settings['controls'],
            FILTER_VALIDATE_BOOLEAN
        ) ) {
            $html .= "<button class='vjs-big-play-button' type='button' title='Play Video' aria-disabled='false'>";
            $html .= "  <span class='vjs-icon-placeholder' aria-hidden='true'></span>";
            $html .= "  <span class='vjs-control-text' aria-live='polite'>Play Video</span>";
            $html .= "</button>";
        }

        return $html;
    }

    /**
     * A setting is enabled or not?
     * 
     * @param string $setting The name of the slide setting. e.g. 'autoPlay'
     * 
     * @return boolean
     */
    private function get_setting_state( $setting )
    {
        return isset( $this->slide_settings[$setting] ) && filter_var(
            $this->slide_settings[$setting],
            FILTER_VALIDATE_BOOLEAN
        ) ? true : false;
    }

    /**
     * Pause videos when the slide is changed
     *
     * @param array $options - current slideshow options
     * @param int $slider_id - current slideshow ID
     * @return array
     */
    public function get_flex_slider_parameters( $options, $slider_id )
    {
        /* This is for the slideshow autoplay, not the video autoplay.
         * This will play the slideshow when the video is paused
         */
        $autoplay = filter_var( $this->settings['autoPlay'], FILTER_VALIDATE_BOOLEAN ) ?
            "player.on('ended', function() {
                $('#metaslider_{$slider_id}').data('flexslider').manualPause = false;
                $('#metaslider_{$slider_id}').flexslider('next');
            	$('#metaslider_{$slider_id}').flexslider('play');
			});" : '';

        // When the slideshow is loaded / first slide
        $options['start'] = isset( $options['start'] ) ? $options['start'] : array();
        $options['start'] = array_merge($options['start'], array(
            "$('#metaslider_{$slider_id} .local-video').each(function(i) {
                var local_video = $(this);
                var id = 'ms_videojs_' + $(this).data('id');
                var active = local_video.parent('.flex-active-slide').length;
                var autoplay = local_video.data('autoplay') && active ? true : false;
                var lazyload = local_video.data('lazyLoad');
                var eventType = lazyload ? 'click' : 'metaslider/load-local-video';

                local_video.on(eventType, function(){

                    if(!$(this).find('video').length) {
                        $(this).append('<video class=\"video-js\" id=\"ms_videojs_' + $(this).data('id') + '\"></video>');
                    } 

                    var player = videojs(id, {
                        controls: $(this).data('controls'),
                        muted: $(this).data('mute'),
                        poster: $(this).data('poster'),
                        fill: true,
                        loop: $(this).data('loop'),
                        sources: [{
                            src: $(this).data('url'),
                            type:$(this).data('type')
                        }]
                    });
    
                    if (active && autoplay) {
                        $('#metaslider_{$slider_id}').flexslider('pause');
                        player.options_.autoplay = autoplay;
                        player.muted(true);
                    }
                    
                    player.on('loadedmetadata', function() {
                        if(eventType === 'click') player.play();
                    });
                    player.on('play', function() {
                        $('#metaslider_{$slider_id}').flexslider('pause');
                        $('#metaslider_{$slider_id}').data('flexslider').manualPause = true;
                        $('#metaslider_{$slider_id}').data('flexslider').manualPlay = false;
                    });
                    {$autoplay}
                });
                if (lazyload && autoplay) $(this).trigger('click');
                lazyload || $(this).trigger('metaslider/load-local-video');          
            });"
        ) );

        /* Before a slide transitions
         * Access the player already created in start event globally with videojs.players[id] 
         * to not generate new video instances of the same video */
        $options['before'] = isset( $options['before'] ) ? $options['before'] : array();
        $options['before'] = array_merge( $options['before'], array(
            "$('#metaslider_{$slider_id} .flex-active-slide .local-video').each(function(i) {
                var id = 'ms_videojs_' + $(this).data('id');

                if(typeof videojs.players[id] === 'undefined') return;

                var player = videojs.players[id];
                if ($(this).data('mute')) player.muted(true);
                player.pause();
            });"
        ) );

        /* After a slide transitions
         * Access the player already created in start event globally with videojs.players[id] 
         * to not generate new video instances of the same video */
        $options['after'] = isset( $options['after'] ) ? $options['after'] : array();
        $options['after'] = array_merge($options['after'], array(
            "$('#metaslider_{$slider_id} .flex-active-slide .local-video').each(function(i) {
                var id = 'ms_videojs_' + $(this).data('id');
                var autoplay = $(this).data('autoplay');
                var lazyload = $(this).data('lazyLoad');

                if(autoplay) {
                    $('#metaslider_{$slider_id}').flexslider('pause');
                    
                    if (lazyload && typeof videojs.players[id] === 'undefined') {
                        $(this).trigger('click');
                    } else {
                        var player = videojs.players[id];
                        player.muted(true);
                        player.play();
                    }
                }
            });"
        ) );

        // we don't want this filter hanging around if there's more than one slideshow on the page
        remove_filter( 'metaslider_flex_slider_parameters', array( $this, 'get_flex_slider_parameters' ) );
        return $options;
    }
}
